@@ -15,12 +15,28 @@
 
 #define DESKTOP_CMD_EXEC_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE (obj, DESKTOP_CMD_EXEC_TYPE, DesktopCmdExecPrivate))
 
+//main settings window
 #define NON_GTK_RESPONSE_ADD_CMD 10
-#define NON_GTK_RESPONSE_DEL_CMD 20
+#define NON_GTK_RESPONSE_EDIT_CMD 20
+//save is GTK_RESPONSE_OK
 
-void desktop_cmd_exec_write_settings (DesktopCmdExec *self);
+//Add/Edit/Delete window
+#define NON_GTK_RESPONSE_DELETE_CMD 30
+//save is GTK_RESPONSE_OK
+
+void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean reset);
 gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self);
 
+//Dialogs prototypes:
+void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new);
+void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self);
+
+//Auxiliary functions' prototypes:
+void AddCommand(DesktopCmdExec *self, gchar * s_title, gchar * s_command);
+void EditCommand(DesktopCmdExec *self, gchar * s_title, gchar * s_command, int index);
+void DelCommand(DesktopCmdExec *self, int index);
+guint GetSeconds(guint index);
+
 struct _DesktopCmdExecPrivate
 {
 
@@ -36,18 +52,24 @@ struct _DesktopCmdExecPrivate
 	/* new */
 	gchar ** c_titles;
 	gchar ** c_commands;
-	gint c_size;
-	gint c_current;
+	guint c_size;
+	guint c_current;
 	
-	gboolean c1_updOnClick;
-	gboolean c1_updOnDesktop;
-	
-	guint eventID;
+	gboolean updOnClick;
+	gboolean updOnDesktop;
+
+	//Update on timer data
+	guint delaySeconds;
+	guint delayIndex;//index of delaySeconds in the array defined inside "desktop_cmd_exec_settings"
+
+	//timer process id; used to stop when closing app or changing delay
+	guint updateTimerID;
 };
 
 HD_DEFINE_PLUGIN_MODULE (DesktopCmdExec, desktop_cmd_exec, HD_TYPE_HOME_PLUGIN_ITEM);
 
 
+//Settings file functions
 void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 {
 // 	g_warning("desktop_cmd_exec_read_settings ");
@@ -55,18 +77,21 @@ void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 	gboolean fileExists;
 	GKeyFile *keyFile;
 
+	gboolean gotAnErrorFlag;
+	gotAnErrorFlag = FALSE;
+
 	keyFile = g_key_file_new();
 	filename = g_strconcat (HOME_DIR, "/.desktop_cmd_exec", NULL);
 	fileExists = g_key_file_load_from_file (keyFile, filename, G_KEY_FILE_KEEP_COMMENTS, NULL);
 
+
 	if (fileExists) {
 		GError *error=NULL;
 
 		//BEGIN get list
 		self->priv->c_commands = (gchar **) g_key_file_get_string_list (keyFile, "config", "c_commands", &(self->priv->c_size) ,&error);
 		if (error) {
-			self->priv->c_commands = NULL;
-			self->priv->c_size = 0;
+			gotAnErrorFlag = TRUE;
 			g_error_free (error);
 			error = NULL;
 		}
@@ -74,15 +99,14 @@ void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 		gint consistencyCheck = -1;
 		self->priv->c_titles = g_key_file_get_string_list (keyFile, "config", "c_titles", &consistencyCheck ,&error);
 		if (error) {
-			self->priv->c_titles = NULL;
-			consistencyCheck = 0;
+			gotAnErrorFlag = TRUE;
 			g_error_free (error);
 			error = NULL;
 		}
 
 		self->priv->c_current = g_key_file_get_integer (keyFile, "config", "c_current", &error);
 		if (error) {
-			self->priv->c_current = 0;
+			gotAnErrorFlag = TRUE;
 			g_error_free (error);
 			error = NULL;
 		}
@@ -90,51 +114,64 @@ void desktop_cmd_exec_read_settings ( DesktopCmdExec *self )
 		if(consistencyCheck != self->priv->c_size)
 		{
 			g_warning("Settings file corrupted!");
-			self->priv->c_commands = NULL;
-			self->priv->c_titles = NULL;
-			self->priv->c_size = 0;
-			self->priv->c_current = 0;
-		}
-		else if (consistencyCheck == 0)
-		{
-			g_warning("No settings file, creating one");
+			gotAnErrorFlag = TRUE;
 		}
 		//END
 
 
-		self->priv->c1_updOnClick = g_key_file_get_boolean (keyFile, "config", "c1_updOnClick", &error);
+		//BEGIN update policy
+		self->priv->updOnClick = g_key_file_get_boolean (keyFile, "config", "updOnClick", &error);
 		if (error) {
-			self->priv->c1_updOnClick = TRUE;
+
+			gotAnErrorFlag = TRUE;
 			g_error_free (error);
 			error = NULL;
 		}
 
-		self->priv->c1_updOnDesktop = g_key_file_get_boolean (keyFile, "config", "c1_updOnDesktop", &error);
+		self->priv->updOnDesktop = g_key_file_get_boolean (keyFile, "config", "updOnDesktop", &error);
 		if (error) {
-			self->priv->c1_updOnDesktop = TRUE;
+
+			gotAnErrorFlag = TRUE;
 			g_error_free (error);
 			error = NULL;
 		}
 
-	} else {
-		self->priv->c1_updOnClick = TRUE;
-		self->priv->c1_updOnDesktop = TRUE;
+		self->priv->delayIndex = (guint) g_key_file_get_integer (keyFile, "config", "delayIndex", &error);
+		if (error) {
+
+			gotAnErrorFlag = TRUE;
+			g_error_free (error);
+			error = NULL;
+		}
+		else
+			self->priv->delaySeconds = GetSeconds(self->priv->delayIndex);
+		
+		//END
+		g_key_file_free (keyFile);
+		g_free (filename);
+	}
+	else
+		gotAnErrorFlag = TRUE;
+
+	if(gotAnErrorFlag)
+	{
+// 		g_warning ("Something happened... resetting all");
+		self->priv->updOnClick = TRUE;
+		self->priv->updOnDesktop = TRUE;
+		self->priv->delayIndex = 0;
+		self->priv->delaySeconds = 0;
 
 		self->priv->c_commands = NULL;
 		self->priv->c_titles = NULL;
 		self->priv->c_size = 0;
 		self->priv->c_current = 0;
-		
-		desktop_cmd_exec_write_settings(self);
+		desktop_cmd_exec_write_settings(self, TRUE);
 	}
-
-	g_key_file_free (keyFile);
-	g_free (filename);
 }
 
-void desktop_cmd_exec_write_settings (DesktopCmdExec *self)
+void desktop_cmd_exec_write_settings (DesktopCmdExec *self, gboolean reset)
 {
-// 	g_warning ("desktop_cmd_exec_write_settings");
+//  	g_warning ("desktop_cmd_exec_write_settings");
 	GKeyFile *keyFile;
 	gchar *fileData;
 	FILE *iniFile;
@@ -143,32 +180,42 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self)
 	
 	keyFile = g_key_file_new();
 
-	if( (!self->priv->c_commands) || self->priv->c_size == 0)
+	if(reset)
 	{
 // 		g_warning("records empty, filling with pre-loaded vals");
 		//records empty, filling with pre-loaded vals
 		self->priv->c_size = 4;
-		gchar * p_titles[] = { "Uptime", "Battery", "Uptime(sb)", "home size(sb)"};
-		gchar * p_commands[] = { "uptime | awk 'NF == 7 {print $2 \" \" $3}; NF == 11 {print $3 \" \" $4 \" \" $5}'",
-		"lshal | grep battery.reporting | awk -F. '{print $3}' | awk '$1 == \"current\" { curval = $3}; $1 == \"design\" {print curval \"/\" $3}'",
-		"cat /proc/uptime | awk '{print $1}'",
-		"ls -lha | awk '$1 == \"total\" {print $2}'"};
-		self->priv->c_titles = p_titles;
-		self->priv->c_commands = p_commands;
+		gchar * p_titles[] = { "Uptime:", "Battery(%):", "Battery(mAh):", "Boot Reason:", NULL};
+		gchar * p_commands[] = {
+		"uptime | awk 'NF == 7 {print $2 \" \" $3}; NF == 8 {print $2 \" \" $3}; NF == 11 {print $3 \" \" $4 \" \" $5}'",
+		"hal-device | grep percen | awk '{print $3 \"%\"}'",
+		"hal-device | grep battery.reporting | awk -F. '{print $3}' | awk '$1 == \"current\" { curval = $3}; $1 == \"design\" {print curval \"/\" $3}'",
+		"cat /proc/bootreason", NULL};
+
+		//clean possible oldies
+		g_strfreev(self->priv->c_titles);
+		g_strfreev(self->priv->c_commands);
+
+		//assign new from stack values
+		self->priv->c_titles = g_strdupv(p_titles);
+		self->priv->c_commands = g_strdupv(p_commands);
+		
 		self->priv->c_current = 2;
 
+		self->priv->updOnClick = TRUE;
+		self->priv->updOnDesktop = TRUE;
+		self->priv->delayIndex = 0;
+		
+// 		g_warning("Diiiiid it");
 	}
 	
 	g_key_file_set_string_list(keyFile, "config", "c_titles", (const gchar **)(self->priv->c_titles),self->priv->c_size);
 	g_key_file_set_string_list(keyFile, "config", "c_commands", (const gchar **)(self->priv->c_commands),self->priv->c_size);
 	g_key_file_set_integer(keyFile, "config", "c_current", self->priv->c_current);
-	
-// 	g_warning("Size %d, Curr %d", self->priv->c_size, self->priv->c_current);
 
-// 	g_warning("Saved::! %s", self->priv->c_titles[self->priv->c_current]);
-
-	g_key_file_set_boolean (keyFile, "config", "c1_updOnClick", self->priv->c1_updOnClick);
-	g_key_file_set_boolean (keyFile, "config", "c1_updOnDesktop", self->priv->c1_updOnDesktop);
+	g_key_file_set_boolean (keyFile, "config", "updOnClick", self->priv->updOnClick);
+	g_key_file_set_boolean (keyFile, "config", "updOnDesktop", self->priv->updOnDesktop);
+	g_key_file_set_integer (keyFile, "config", "delayIndex", self->priv->delayIndex);
 	
 	filename = g_strconcat (HOME_DIR, "/.desktop_cmd_exec", NULL);
 	fileData = g_key_file_to_data (keyFile, &size, NULL);
@@ -178,13 +225,17 @@ void desktop_cmd_exec_write_settings (DesktopCmdExec *self)
 	g_key_file_free (keyFile);
 	g_free (fileData);
 	g_free (filename);
-	desktop_cmd_exec_read_settings (self);
+	if(reset)
+		desktop_cmd_exec_read_settings(self);
 }
 
+
+
+//widget/touchscreen interaction callbacks
 void desktop_cmd_exec_button_press (GtkWidget *widget, GdkEventButton *event, DesktopCmdExec *self)
 {
 // 	g_warning ("desktop_cmd_exec_button_press");
-	if (self->priv->c1_updOnClick) {
+	if (self->priv->updOnClick) {
 		self->priv->isPressed = TRUE;
 		gtk_widget_queue_draw (GTK_WIDGET (self));
 	}
@@ -193,7 +244,7 @@ void desktop_cmd_exec_button_press (GtkWidget *widget, GdkEventButton *event, De
 void desktop_cmd_exec_button_release (GtkWidget *widget, GdkEventButton *event, DesktopCmdExec *self)
 {
 // 	g_warning ("desktop_cmd_exec_button_release");
-	if (self->priv->c1_updOnClick) {
+	if (self->priv->updOnClick) {
 		self->priv->isPressed = FALSE;
 		gtk_widget_queue_draw (GTK_WIDGET (self));
 		desktop_cmd_exec_update_content(self);
@@ -203,12 +254,23 @@ void desktop_cmd_exec_button_release (GtkWidget *widget, GdkEventButton *event,
 void desktop_cmd_exec_leave_event (GtkWidget *widget, GdkEventCrossing *event, DesktopCmdExec *self)
 {
 // 	g_warning ("desktop_cmd_exec_leave_event");
-	if (self->priv->c1_updOnClick) {
+	if (self->priv->updOnClick) {
 		self->priv->isPressed = FALSE;
 		gtk_widget_queue_draw (GTK_WIDGET (self));
 	}
 }
 
+static void desktop_cmd_exec_check_desktop (GObject *gobject, GParamSpec *pspec, DesktopCmdExec *self)
+{
+	if (self->priv->updOnDesktop) {
+		desktop_cmd_exec_update_content(self);
+	}
+	// 	g_warning ("desktop_cmd_exec_check_desktop");
+}
+
+
+
+//Content/widget creation/update
 void desktop_cmd_exec_content_create (DesktopCmdExec *self)
 {
 // 	g_warning ("desktop_cmd_exec_content_create");
@@ -245,6 +307,8 @@ void desktop_cmd_exec_content_create (DesktopCmdExec *self)
 	gtk_widget_show (self->priv->cmdTitle_lb);
 }
 
+
+
 gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self)
 {
 // 	g_warning ("desktop_cmd_exec_update_content");
@@ -275,176 +339,17 @@ gboolean desktop_cmd_exec_update_content (DesktopCmdExec *self)
 	if (!found) {
 		gtk_label_set_text (GTK_LABEL (self->priv->cmdResult_lb), "Invalid Command");
 	}
+
+	if( (self->priv->updateTimerID == 0) && (self->priv->delaySeconds > 0))
+	{
+		self->priv->updateTimerID = g_timeout_add_seconds (self->priv->delaySeconds, (GSourceFunc)desktop_cmd_exec_update_content, self);
+	}
 	
 	return found;
 }
 
-void desktop_cmd_exec_add_cmd ( DesktopCmdExec *self)
-{
-	GtkWidget *dialog = gtk_dialog_new_with_buttons ("Add custom command", NULL, 0,
-																									 _("wdgt_bd_save"), GTK_RESPONSE_ACCEPT, NULL);
-	GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
-	
-	GtkWidget *titleBox = gtk_hbox_new (FALSE, 0);
-	GtkWidget *titleLabel = gtk_label_new ("Title:");
-	gtk_misc_set_alignment (GTK_MISC (titleLabel), 0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (titleLabel), HILDON_MARGIN_DOUBLE, 0);
-	gtk_size_group_add_widget (group, titleLabel);
-
-	GtkWidget *titleEntry = hildon_entry_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-	gtk_box_pack_start (GTK_BOX (titleBox), titleLabel, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (titleBox), titleEntry, TRUE, TRUE, 0);
-
-	gtk_container_add (GTK_CONTAINER (content_area), titleBox);
 
 
-	GtkWidget *commandBox = gtk_hbox_new (FALSE, 0);
-	GtkWidget *commandLabel = gtk_label_new ("Command:");
-	gtk_misc_set_alignment (GTK_MISC (commandLabel), 0, 0.5);
-	gtk_misc_set_padding (GTK_MISC (commandLabel), HILDON_MARGIN_DOUBLE, 0);
-	gtk_size_group_add_widget (group, commandLabel);
-
-	GtkWidget *commandEntry = hildon_entry_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-	gtk_box_pack_start (GTK_BOX (commandBox), commandLabel, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (commandBox), commandEntry, TRUE, TRUE, 0);
-
-	gtk_container_add (GTK_CONTAINER (content_area), commandBox);
-	gtk_widget_show_all (dialog);
-
-
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
-
-// 		g_warning("Adding: %s", g_strdup (hildon_entry_get_text (HILDON_ENTRY (titleEntry))));
-// 		g_warning("cmd: %s", g_strdup (hildon_entry_get_text (HILDON_ENTRY (commandEntry))));
-		
-		gchar ** newTitles = malloc( sizeof(gchar*)*(self->priv->c_size+1) );
-		gchar ** newCommands = malloc( sizeof(gchar*)*(self->priv->c_size+1) );
-		
-		int i;
-		for(i=0;i<self->priv->c_size;i++)
-		{
-			newTitles[i] = g_strdup(self->priv->c_titles[i]);
-			newCommands[i] = g_strdup(self->priv->c_commands[i]);
-		}
-		
-		newTitles[self->priv->c_size] = g_strdup (hildon_entry_get_text (HILDON_ENTRY (titleEntry)));
-		newCommands[self->priv->c_size] = g_strdup (hildon_entry_get_text (HILDON_ENTRY (commandEntry)));
-
-		self->priv->c_current = self->priv->c_size;
-		self->priv->c_size += 1;
-
-
-
-		self->priv->c_titles = newTitles;
-		self->priv->c_commands = newCommands;
-
-		desktop_cmd_exec_write_settings (self);
-		
-		desktop_cmd_exec_update_content (self);
-	}
-	
-	gtk_widget_destroy (dialog);
-}
-
-void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
-{
-// 	g_warning ("desktop_cmd_exec_settings");
-
-	int settingsRunning = 1;
-	while(settingsRunning)
-	{
-		GtkWidget *dialog = gtk_dialog_new_with_buttons ("Desktop Command Execution Widget Settings", NULL, 0,
-																										_("wdgt_bd_save"), GTK_RESPONSE_ACCEPT,
-																										"Add Entry", NON_GTK_RESPONSE_ADD_CMD,
-																										//"Delete Entry", NON_GTK_RESPONSE_DEL_CMD,
-																										 NULL);
-
-		GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-		GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
-
-		GtkWidget *selector = hildon_touch_selector_new_text ();
-		int i;
-		for(i=0;i<self->priv->c_size;i++)
-		{
-			hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), self->priv->c_titles[i]);
-		}
-		hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, self->priv->c_current);
-
-
-		GtkWidget * cmdSelector = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
-		hildon_button_set_title (HILDON_BUTTON (cmdSelector), "Preloaded Commands");
-		hildon_button_add_title_size_group (HILDON_BUTTON (cmdSelector), group);
-		hildon_button_set_alignment (HILDON_BUTTON (cmdSelector), 0, 0.5, 0, 0);
-		hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (cmdSelector), HILDON_TOUCH_SELECTOR (selector));
-		gtk_container_add (GTK_CONTAINER (content_area), cmdSelector);
-
-		GtkHBox *hbox_upds = GTK_HBOX (gtk_hbox_new (FALSE, 0));
-
-		GtkWidget *toggleOnClick = hildon_gtk_toggle_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-		gtk_button_set_label (GTK_BUTTON (toggleOnClick), "@Click");
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggleOnClick), self->priv->c1_updOnClick);
-		gtk_container_add (GTK_CONTAINER (hbox_upds), toggleOnClick);
-
-		GtkWidget *toggleOnDesktop = hildon_gtk_toggle_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
-		gtk_button_set_label (GTK_BUTTON (toggleOnDesktop), "@Desktop");
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggleOnDesktop), self->priv->c1_updOnDesktop);
-		gtk_container_add (GTK_CONTAINER (hbox_upds), toggleOnDesktop);
-
-		GtkWidget *updBox = gtk_hbox_new (FALSE, 0);
-		GtkWidget *updLabel = gtk_label_new ("Updade:");
-		gtk_misc_set_alignment (GTK_MISC (updLabel), 0, 0.5);
-		gtk_misc_set_padding (GTK_MISC (updLabel), HILDON_MARGIN_DOUBLE, 0);
-		gtk_size_group_add_widget (group, updLabel);
-		gtk_box_pack_start (GTK_BOX (updBox), updLabel, FALSE, FALSE, 0);
-		gtk_box_pack_start (GTK_BOX (updBox), GTK_WIDGET (hbox_upds), TRUE, TRUE, 0);
-
-		gtk_container_add (GTK_CONTAINER (content_area), updBox);
-
-		gtk_widget_show_all (dialog);
-
-		int setDialogReturn = 0;
-
-		setDialogReturn = gtk_dialog_run (GTK_DIALOG (dialog));
-		if (setDialogReturn == GTK_RESPONSE_ACCEPT)
-		{
-			int cmdSel = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
-// 			g_warning("Selected: %d", cmdSel);
-
-			if(cmdSel < self->priv->c_size && cmdSel >= 0)
-			{
-				self->priv->c_current = (gint) cmdSel;
-			}
-			self->priv->c1_updOnClick = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggleOnClick));
-			self->priv->c1_updOnDesktop = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggleOnDesktop));
-
-			desktop_cmd_exec_write_settings (self);
-			desktop_cmd_exec_update_content (self);
-			settingsRunning = 0;
-		}
-		else if(setDialogReturn == NON_GTK_RESPONSE_ADD_CMD)
-		{
-			desktop_cmd_exec_add_cmd(self);
-			
-			for(i=0;i<self->priv->c_size;i++)
-			{
-				hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), self->priv->c_titles[i]);
-			}
-			hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, self->priv->c_current);
-		}
-		else
-			settingsRunning = 0;
-	gtk_widget_destroy (dialog);
-	}
-}
-
-static void desktop_cmd_exec_check_desktop (GObject *gobject, GParamSpec *pspec, DesktopCmdExec *self)
-{
-	if (self->priv->c1_updOnClick) {
-		desktop_cmd_exec_update_content(self);
-	}
-// 	g_warning ("desktop_cmd_exec_check_desktop");
-}
 
 static void desktop_cmd_exec_dispose (GObject *object)
 {
@@ -459,8 +364,8 @@ static void desktop_cmd_exec_finalize (GObject *object)
 // 	g_warning ("desktop_cmd_exec_finalize");
 	DesktopCmdExec *self = DESKTOP_CMD_EXEC (object);
 
-	if (self->priv->eventID) {
-		g_source_remove (self->priv->eventID);
+	if (self->priv->updateTimerID) {
+		g_source_remove (self->priv->updateTimerID);
 	}
 
 	g_strfreev(self->priv->c_titles);
@@ -509,14 +414,11 @@ static gboolean desktop_cmd_exec_expose_event (GtkWidget *widget, GdkEventExpose
 
 	cairo_move_to(cr, x + radius, y);
 	cairo_line_to(cr, x + width - radius, y);
-	cairo_curve_to(cr, x + width - radius, y, x + width, y, x + width,
-				y + radius);
+	cairo_curve_to(cr, x + width - radius, y, x + width, y, x + width,y + radius);
 	cairo_line_to(cr, x + width, y + height - radius);
-	cairo_curve_to(cr, x + width, y + height - radius, x + width,
-				y + height, x + width - radius, y + height);
+	cairo_curve_to(cr, x + width, y + height - radius, x + width,y + height, x + width - radius, y + height);
 	cairo_line_to(cr, x + radius, y + height);
-	cairo_curve_to(cr, x + radius, y + height, x, y + height, x,
-				y + height - radius);
+	cairo_curve_to(cr, x + radius, y + height, x, y + height, x,y + height - radius);
 	cairo_line_to(cr, x, y + radius);
 	cairo_curve_to(cr, x, y + radius, x, y, x + radius, y);
 
@@ -557,29 +459,28 @@ static void desktop_cmd_exec_init (DesktopCmdExec *self)
 {
 // 	g_warning ("desktop_cmd_exec_init");
 	self->priv = DESKTOP_CMD_EXEC_GET_PRIVATE (self);
-	self->priv->eventID = 0;
+	self->priv->updateTimerID = 0;
 	self->priv->isPressed = FALSE;
 	
-	
 	hd_home_plugin_item_set_settings (&self->parent, TRUE);
 	g_signal_connect (&self->parent, "show-settings", G_CALLBACK (desktop_cmd_exec_settings), self);
 
-	gtk_window_set_default_size (GTK_WINDOW (self), 315, 56);
+	gtk_window_set_default_size (GTK_WINDOW (self), 800, 40);
 	
 	self->priv->homeWidget = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (self), self->priv->homeWidget);
 	gtk_widget_show (self->priv->homeWidget);
 
 	GdkGeometry hints;
-	hints.min_width = 515;
+	hints.min_width = 800;
 	hints.min_height = 40;
-	hints.max_width = 515;
+	hints.max_width = 800;
 	hints.max_height = 40;
 	gtk_window_set_geometry_hints (GTK_WINDOW (self), self->priv->homeWidget, &hints, GDK_HINT_MIN_SIZE | GDK_HINT_MAX_SIZE);
 	
 	desktop_cmd_exec_read_settings (self);
-	gtk_widget_set_size_request (GTK_WIDGET (self), 515, 40);
-	gtk_window_resize (GTK_WINDOW (self), 515, 40);
+	gtk_widget_set_size_request (GTK_WIDGET (self), 800, 40);
+	gtk_window_resize (GTK_WINDOW (self), 800, 40);
 
 	
 	desktop_cmd_exec_content_create (self);
@@ -592,3 +493,308 @@ DesktopCmdExec* desktop_cmd_exec_new (void)
 {
 	return g_object_new (DESKTOP_CMD_EXEC_TYPE, NULL);
 }
+
+
+//Separate window dialogs
+void desktop_cmd_exec_settings (HDHomePluginItem *hitem, DesktopCmdExec *self)
+{
+	// 	g_warning ("desktop_cmd_exec_settings");
+	
+	int settingsRunning = 1;
+	while(settingsRunning)
+	{
+		GtkWidget *dialog = gtk_dialog_new_with_buttons ("Desktop Command Execution Widget Settings", NULL, 0,
+																										 _("wdgt_bd_save"), GTK_RESPONSE_ACCEPT,
+																										 "Add Cmd", NON_GTK_RESPONSE_ADD_CMD,
+																										 "Edit Cmd", NON_GTK_RESPONSE_EDIT_CMD,
+																										 NULL);
+
+		GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+		GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
+
+		GtkWidget *cmdSelectionLabel = gtk_label_new ("");
+		gtk_label_set_markup (GTK_LABEL (cmdSelectionLabel), "<small>Command Selection:</small>");
+		gtk_container_add (GTK_CONTAINER (content_area), cmdSelectionLabel);
+
+		//BEGIN Command Selector
+		
+		GtkWidget *selector = hildon_touch_selector_new_text ();
+		int i;
+		for(i=0;i<self->priv->c_size;i++)
+		{
+			hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (selector), self->priv->c_titles[i]);
+		}
+		hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (selector), 0, self->priv->c_current);
+
+
+		GtkWidget * cmdSelector = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
+		hildon_button_set_title (HILDON_BUTTON (cmdSelector), "Commands: ");
+		hildon_button_add_title_size_group (HILDON_BUTTON (cmdSelector), group);
+		hildon_button_set_alignment (HILDON_BUTTON (cmdSelector), 0, 0.5, 0, 0);
+		hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (cmdSelector), HILDON_TOUCH_SELECTOR (selector));
+		gtk_container_add (GTK_CONTAINER (content_area), cmdSelector);
+		//END
+
+		//BEGIN Update Policy
+		GtkWidget *updPolicyLabel = gtk_label_new ("");
+		gtk_label_set_markup (GTK_LABEL (updPolicyLabel), "<small>Update Policy:</small>");
+		gtk_container_add (GTK_CONTAINER (content_area), updPolicyLabel);
+		
+		GtkWidget *checkBtClick = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+		gtk_button_set_label (GTK_BUTTON (checkBtClick), "Update when clicked");
+		gtk_container_add (GTK_CONTAINER (content_area), checkBtClick);
+		hildon_check_button_set_active (HILDON_CHECK_BUTTON (checkBtClick), self->priv->updOnClick);
+
+		GtkWidget *checkBtDesktop = hildon_check_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+		gtk_button_set_label (GTK_BUTTON (checkBtDesktop), "Update when switched to the desktop");
+		gtk_container_add (GTK_CONTAINER (content_area), checkBtDesktop);
+		hildon_check_button_set_active (HILDON_CHECK_BUTTON (checkBtDesktop), self->priv->updOnDesktop);
+
+		GtkWidget *intervalSelector = hildon_touch_selector_new_text ();
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "0");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "30 Seconds");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "1 Minute");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "5 Minutes");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "30 Minutes");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "1 Hour");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "6 Hours");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "12 Hours");
+		hildon_touch_selector_append_text (HILDON_TOUCH_SELECTOR (intervalSelector), "Daily");
+		
+		hildon_touch_selector_set_active (HILDON_TOUCH_SELECTOR (intervalSelector), 0, self->priv->delayIndex);
+		
+		GtkWidget * intervalSelBt = hildon_picker_button_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_HORIZONTAL);
+		hildon_button_set_title (HILDON_BUTTON (intervalSelBt), "Update Interval: ");
+		hildon_button_add_title_size_group (HILDON_BUTTON (intervalSelBt), group);
+		hildon_button_set_alignment (HILDON_BUTTON (intervalSelBt), 0, 0.5, 0, 0);
+		hildon_picker_button_set_selector (HILDON_PICKER_BUTTON (intervalSelBt), HILDON_TOUCH_SELECTOR (intervalSelector));
+		gtk_container_add (GTK_CONTAINER (content_area), intervalSelBt);
+
+		
+		GtkWidget *updIntervalLabel = gtk_label_new ("");
+		gtk_label_set_markup (GTK_LABEL (updIntervalLabel), "<small>'0' disables periodic updating</small>");
+		gtk_container_add (GTK_CONTAINER (content_area), updIntervalLabel);
+		//END
+
+		gtk_widget_show_all (dialog);
+
+		//BEGIN result processing
+
+		int setDialogReturn = 0;
+
+		setDialogReturn = gtk_dialog_run (GTK_DIALOG (dialog));
+		if (setDialogReturn == GTK_RESPONSE_ACCEPT)
+		{
+			int cmdSel = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
+			if(cmdSel < self->priv->c_size && cmdSel >= 0)
+			{
+				self->priv->c_current = (gint) cmdSel;
+			}
+			self->priv->updOnClick = hildon_check_button_get_active (HILDON_CHECK_BUTTON (checkBtClick));
+			self->priv->updOnDesktop = hildon_check_button_get_active (HILDON_CHECK_BUTTON (checkBtDesktop));
+			
+			self->priv->delayIndex = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (intervalSelector), 0);
+			self->priv->delaySeconds = GetSeconds(self->priv->delayIndex);
+			
+			if (self->priv->updateTimerID != 0) {
+				g_source_remove (self->priv->updateTimerID);
+				self->priv->updateTimerID = 0;
+			}
+			else
+			
+
+			desktop_cmd_exec_write_settings (self, FALSE);
+			desktop_cmd_exec_update_content (self);
+			settingsRunning = 0;
+		}
+		else if(setDialogReturn == NON_GTK_RESPONSE_ADD_CMD)
+		{
+			desktop_cmd_exec_edit_add_dialog(self, 1);
+		}
+		else if(setDialogReturn == NON_GTK_RESPONSE_EDIT_CMD)
+		{
+			int cmdSel = hildon_touch_selector_get_active (HILDON_TOUCH_SELECTOR (selector), 0);
+			if(cmdSel < self->priv->c_size && cmdSel >= 0)
+			{
+				self->priv->c_current = (gint) cmdSel;
+			}
+			desktop_cmd_exec_edit_add_dialog(self,0);
+		}
+		else
+			settingsRunning = 0;
+		
+		gtk_widget_destroy (dialog);
+
+		//END
+	}
+}
+
+void desktop_cmd_exec_edit_add_dialog ( DesktopCmdExec *self, gboolean new)
+{
+	GtkWidget *dialog;
+	gchar * title;
+	if(new)
+		title = g_strconcat ("Add new command", NULL);
+	else
+		title = g_strconcat ("Edit ", self->priv->c_titles[self->priv->c_current] ," command", NULL);
+
+	if(new)
+		dialog = gtk_dialog_new_with_buttons (title, NULL, 0, _("wdgt_bd_save"), GTK_RESPONSE_ACCEPT, NULL);
+	else
+		dialog = gtk_dialog_new_with_buttons (title, NULL, 0, _("wdgt_bd_save"), GTK_RESPONSE_ACCEPT, _("wdgt_bd_delete"), NON_GTK_RESPONSE_DELETE_CMD, NULL);
+	
+	GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+	GtkSizeGroup *group = GTK_SIZE_GROUP (gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL));
+
+	//BEGIN title
+	GtkWidget *titleBox = gtk_hbox_new (FALSE, 0);
+	
+	GtkWidget *titleLabel = gtk_label_new ("Title:");
+	gtk_misc_set_alignment (GTK_MISC (titleLabel), 0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (titleLabel), HILDON_MARGIN_DOUBLE, 0);
+	gtk_size_group_add_widget (group, titleLabel);
+	
+	GtkWidget *titleEntry = hildon_entry_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+	gtk_box_pack_start (GTK_BOX (titleBox), titleLabel, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (titleBox), titleEntry, TRUE, TRUE, 0);
+	if(!new)
+		hildon_entry_set_text(HILDON_ENTRY (titleEntry), self->priv->c_titles[self->priv->c_current]);
+	
+	gtk_container_add (GTK_CONTAINER (content_area), titleBox);
+	//END
+
+	//BEGIN command
+	GtkWidget *commandBox = gtk_hbox_new (FALSE, 0);
+	
+	GtkWidget *commandLabel = gtk_label_new ("Command:");
+	gtk_misc_set_alignment (GTK_MISC (commandLabel), 0, 0.5);
+	gtk_misc_set_padding (GTK_MISC (commandLabel), HILDON_MARGIN_DOUBLE, 0);
+	gtk_size_group_add_widget (group, commandLabel);
+	
+	GtkWidget *commandEntry = hildon_entry_new (HILDON_SIZE_AUTO_WIDTH | HILDON_SIZE_FINGER_HEIGHT);
+	gtk_box_pack_start (GTK_BOX (commandBox), commandLabel, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (commandBox), commandEntry, TRUE, TRUE, 0);
+	if(!new)
+		hildon_entry_set_text(HILDON_ENTRY (commandEntry), self->priv->c_commands[self->priv->c_current]);
+	
+	gtk_container_add (GTK_CONTAINER (content_area), commandBox);
+	//END
+
+	gtk_widget_show_all (dialog);
+	int dialogRunResponse = gtk_dialog_run (GTK_DIALOG (dialog));
+	switch(dialogRunResponse)
+	{
+		case GTK_RESPONSE_ACCEPT:
+			if(new)
+			{
+				AddCommand(self, g_strdup(hildon_entry_get_text (HILDON_ENTRY (titleEntry))), g_strdup(hildon_entry_get_text (HILDON_ENTRY (commandEntry))));
+			}
+			else
+			{
+				EditCommand(self, g_strdup(hildon_entry_get_text (HILDON_ENTRY (titleEntry))), g_strdup(hildon_entry_get_text (HILDON_ENTRY (commandEntry))), self->priv->c_current);
+			}
+			break;
+
+		case NON_GTK_RESPONSE_DELETE_CMD:
+			DelCommand(self,self->priv->c_current);
+			break;
+
+	}
+	gtk_widget_destroy (dialog);
+	g_free(title);
+}
+
+
+
+
+
+//Auxiliary functions
+void AddCommand(DesktopCmdExec *self, gchar * s_title, gchar * s_command)
+{
+	gchar ** newTitles = malloc( sizeof(gchar*)*(self->priv->c_size+2) );
+	gchar ** newCommands = malloc( sizeof(gchar*)*(self->priv->c_size+2) );
+	
+	int i;
+	for(i=0;i<self->priv->c_size;i++)
+	{
+		newTitles[i] = g_strdup(self->priv->c_titles[i]);
+		newCommands[i] = g_strdup(self->priv->c_commands[i]);
+	}
+	
+	g_strfreev(self->priv->c_titles);
+	g_strfreev(self->priv->c_commands);
+	
+	newTitles[self->priv->c_size] = s_title;
+	newCommands[self->priv->c_size] = s_command;
+
+	newTitles[self->priv->c_size+1] = NULL;
+	newCommands[self->priv->c_size+1] = NULL;
+	
+	self->priv->c_current = self->priv->c_size;
+	self->priv->c_size += 1;
+	
+	self->priv->c_titles = newTitles;
+	self->priv->c_commands = newCommands;
+}
+
+void EditCommand(DesktopCmdExec *self, gchar * s_title, gchar * s_command, int index)
+{
+	//Clean old
+	g_free(self->priv->c_titles[index]);
+	g_free(self->priv->c_commands[index]);
+
+	self->priv->c_titles[index] = s_title;
+	self->priv->c_commands[index] = s_command;
+}
+
+void DelCommand(DesktopCmdExec *self, int index)
+{
+	gchar ** newTitles = malloc( sizeof(gchar*)*(self->priv->c_size) );
+	gchar ** newCommands = malloc( sizeof(gchar*)*(self->priv->c_size) );
+	g_warning("mallocated %d", self->priv->c_size);
+
+	int i;
+	for(i=0;i<self->priv->c_size;i++)
+	{
+		g_warning("gathering..%d", i);
+		//index is the delete target
+		if(i < index)
+		{
+			newTitles[i] = g_strdup(self->priv->c_titles[i]);
+			newCommands[i] = g_strdup(self->priv->c_commands[i]);
+		}
+		else if(i > index)
+		{
+			newTitles[i-1] = g_strdup(self->priv->c_titles[i]);
+			newCommands[i-1] = g_strdup(self->priv->c_commands[i]);
+			g_warning("wrote at %d",i-1);
+		}
+	}
+	g_warning("ok.. no prob");
+	newTitles[self->priv->c_size-1] = NULL;
+	newCommands[self->priv->c_size-1] = NULL;
+	g_warning("nulled %d", self->priv->c_size-1);
+
+	//clear old
+	g_strfreev(self->priv->c_titles);
+	g_strfreev(self->priv->c_commands);
+	g_warning("cleared ok");
+
+	//assign new
+	self->priv->c_titles = newTitles;
+	self->priv->c_commands = newCommands;
+
+	self->priv->c_current = 0;
+	self->priv->c_size -= 1;//decrement size
+	g_warning("end of del");
+}
+
+guint GetSeconds(guint index)
+{
+	if(index > 0 && index < 9)
+	{
+		guint intervalTimes[] = {0, 30, 60, 300, 1800, 3600, 21600, 43200, 86400};
+		return intervalTimes[index];
+	}
+	return 0;
+}
